import 'dart:async';
import 'dart:io';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:uuid/uuid.dart';

import '../../abg_utils.dart';
import 'db_transfer.dart';

List<ProviderData> providers = [];
ProviderData currentProvider = ProviderData.createEmpty();


LatLng getProviderRouteLatLng(String providerId){ // parent.currentService.providers.isNotEmpty ? parent.currentService.providers[0] : ""
  ProviderData? _provider = getProviderById(providerId);
  if (_provider == null)
    return LatLng(0, 0);
  if (_provider.route.isEmpty)
    return LatLng(0, 0);

  return _provider.route[0];
}

List<LatLng> getProviderRoute(String providerId){ // parent.currentService.providers[0]
  ProviderData? _provider = getProviderById(providerId);
  if (_provider == null)
    return [];
  return _provider.route;
}

Future<ProviderData?> getProviderData(String login) async {
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("provider").where("login", isEqualTo: login).get();
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      if (_data.isNotEmpty){
        return ProviderData.fromJson(result.id, _data);
      }
    }
  }catch(ex){
    return null;
  }
  return null;
}

Future<String> getProviderId(String login) async {
  var ret = "";
  try{
    var querySnapshot = await FirebaseFirestore.instance.collection("provider").where("login", isEqualTo: login).get();
    for (var result in querySnapshot.docs) {
      var data = result.data();
      if (data.isEmpty)
        return "";
      ret = result.id;
    }
  }catch(ex){
    return "";
  }
  return ret;
}

ProviderData? getProviderById(String _providerId) {
  for (var item in providers)
    if (item.id == _providerId)
      return item;
}

String getProviderNameById(String _providerId, String locale) {
  ProviderData? provider = getProviderById(_providerId);
  if (provider == null)
    return "";
  return getTextByLocale(provider.name, locale);
}

String getProviderImageById(String _providerId) {
  if (_providerId.isEmpty)
    return appSettings.customerLogoServer;
  ProviderData? provider = getProviderById(_providerId);
  if (provider == null)
    return "";
  if (provider.gallery.isNotEmpty)
    return provider.gallery[0].serverPath;
  return "";
}

ProviderData? getProviderByEmail(String userEmail) {
  for (var item in providers)
    if (item.login == userEmail)
      return item;
}

Future<UserData?> getProviderUserByEmail(String userEmail) async {
  try{
    List<UserData> _list = await dbGetAllDocumentInTable("listusers", field1: "email", isEqualTo1: userEmail);
    if (_list.isNotEmpty)
      return _list[0];
  }catch(ex){
    dprint("getProviderUserByEmail " + ex.toString());
  }
  return null;
}

Future<String?> saveWorkArea(ProviderData _provider) async {
  try{
    await FirebaseFirestore.instance.collection("provider").doc(_provider.id).set({
      "route": _provider.route.map((i){
        return {'lat': i.latitude, 'lng': i.longitude};
      }).toList()
    }, SetOptions(merge: true));
  }catch(ex){
    return "saveWorkArea " + ex.toString();
  }
  return null;
}

Future<String?> saveProviderArticles(ProviderData _provider) async {
  try{
    await FirebaseFirestore.instance.collection("provider").doc(_provider.id).set({
      "articles": _provider.articles
    }, SetOptions(merge: true));
  }catch(ex){
    return "saveProviderArticles " + ex.toString();
  }
  return null;
}

String getProviderAddress(String id){
  String _address = "";
  for (var item in providers)
    if (item.id == id) {
      _address = item.address;
      break;
    }
  return _address;
}

_initWeekEnd(){
  if (currentProvider.workTime.length != 7){
    currentProvider.workTime = [
      WorkTimeData(id: 0),
      WorkTimeData(id: 1),
      WorkTimeData(id: 2),
      WorkTimeData(id: 3),
      WorkTimeData(id: 4),
      WorkTimeData(id: 5),
      WorkTimeData(id: 6),
    ];
  }
}

setWeekend(bool val, String weekDataComboValue){
  _initWeekEnd();
  for (var item in currentProvider.workTime)
    if (item.id.toString() == weekDataComboValue)
      item.weekend = val;
}

Future<String?> setProviderAvailable(ProviderData provider, bool val) async {
  try{
    await FirebaseFirestore.instance.collection("provider").doc(provider.id).set({
      'available': val
    }, SetOptions(merge: true));
  }catch(ex){
    return "setProviderAvailable " + ex.toString();
  }
  return null;
}

Future<String?> saveProviderFromAdmin() async {
  try{
    categorySave();
    var _data = currentProvider.toJson();
    await FirebaseFirestore.instance.collection("provider").doc(currentProvider.id)
        .set(_data, SetOptions(merge:true));
  }catch(ex){
    return "MainDataProvider save " + ex.toString();
  }
  return null;
}

Future<String?> createProviderFromAdmin(UserData? newProvider, List<UserData> providersRequest) async {
  try{
    var _data = currentProvider.toJson();
    var t = await FirebaseFirestore.instance.collection("provider").add(_data);
    currentProvider.id = t.id;
    providers.add(currentProvider);
    await FirebaseFirestore.instance.collection("settings").doc("main")
        .set({"provider_count": FieldValue.increment(1)}, SetOptions(merge:true));
    if (newProvider != null) {
      await FirebaseFirestore.instance.collection("listusers").doc(newProvider!.id)
          .set({"providerRequest": false}, SetOptions(merge: true));
      providersRequest.remove(newProvider);
      newProvider = null;
      await FirebaseFirestore.instance.collection("settings").doc("main")
          .set({"provider_request_count": FieldValue.increment(-1)}, SetOptions(merge:true));
    }
  }catch(ex){
    return "createProviderFromAdmin " + ex.toString();
  }
  return null;
}

Future<String?> registerProvider(String email, String pass, String _login,
    String providerName, String phone, String desc, String address,
    List<String> category, String pathToImage, List<LatLng> routeForNewProvider) async {
  try {
    final User? user = (await FirebaseAuth.instance.createUserWithEmailAndPassword(
      email: email, password: pass,)).user;
    if (user == null)
      return "User don't create";

    // load image
    var f = Uuid().v4();
    var name = "provider/$f.jpg";
    var firebaseStorageRef = FirebaseStorage.instance.ref().child(name);
    TaskSnapshot s = await firebaseStorageRef.putFile(File(pathToImage));

    await FirebaseFirestore.instance.collection("listusers").doc(user.uid).set({
      "visible": true,
      "providerApp": true,
      "providerRequest" : true,
      "phoneVerified": false,
      "email": user.email,
      "phone": phone,
      "name": _login,
      "date_create" : FieldValue.serverTimestamp(),
      "providerName": providerName,
      "providerAddress": address,
      "providerDesc": desc,
      "providerLogoLocalFile": name,
      "providerLogoServerPath": await s.ref.getDownloadURL(),
      "providerWorkArea": routeForNewProvider.map((i){
        return {'lat': i.latitude, 'lng': i.longitude};
      }).toList(),
      "providerCategory": category
    }, SetOptions(merge:true));

    await FirebaseFirestore.instance.collection("settings").doc("main")
        .set({"provider_request_count": FieldValue.increment(1)}, SetOptions(merge:true));
    await FirebaseFirestore.instance.collection("settings").doc("main")
        .set({"provider_new_request_count": FieldValue.increment(1)}, SetOptions(merge:true));

  }catch(ex){
    return "model registerProvider " + ex.toString();
  }
  return null;
}

loadProvider(Function() callbackLoad) async{
  FirebaseFirestore.instance.collection("provider").snapshots().listen((querySnapshot){
    providers = [];
    for (var result in querySnapshot.docs) {
      var _data = result.data();
      // dprint("Provider $_data");
      var t = ProviderData.fromJson(result.id, _data);
      providers.add(t);
    }
    callbackLoad();
    addStat("provider", querySnapshot.docs.length);
  }).onError((ex){
    messageError(buildContext, "loadProvider " + ex.toString());
  });
}

class ProviderData {
  String id;
  List<StringData> name;
  String phone;
  String www;
  String instagram;
  String telegram;
  List<StringData> desc;
  List<StringData> descTitle;
  String address;
  //String avatar;
  bool visible;
  int unread = 0;
  int all = 0;
  String login;
  String lastMessage = "";
  DateTime lastMessageTime = DateTime.now();
  String chatId = "";

  String imageUpperServerPath = "";
  String imageUpperLocalFile = "";
  String logoServerPath = "";
  String logoLocalFile = "";
  List<ImageData> gallery = [];
  //
  List<WorkTimeData> workTime = [];
  List<String> category = [];
  //
  bool select = false;
  final dataKey = GlobalKey();
  List<LatLng> route;
  double distanceToUser = 0;
  bool available = true; // провайдер доступен или нет. В приложении провайдера устанавливается в Account'e
  String assetUpperImage;
  String assetsLogo;
  List<String> assetsGallery;
  List<String> assetsCategory;
  List<AddonData> addon;
  double tax; // default tax for provider
  List<String> articles;

  //
  bool useMaximumServices; // максимальное количество сервисов
  int maxServices;
  bool useMaxAddonInOneService; // максимальное количество addon в одном сервисе
  int maxAddonInOneService;
  bool useMaximumProducts;  // максимальное количество товаров
  int maxProducts;
  bool useMinPurchaseAmount; // минимальная сумма покупки для этого провайдера в одном заказе
  double minPurchaseAmount;
  bool useMaxPurchaseAmount; // максимальная сумма покупки для этого провайдера в одном заказе
  double maxPurchaseAmount;
  bool acceptPaymentInCash; // принимать оплату наличными

  ProviderData({this.id = "", required this.name, this.visible = true, this.address = "", required this.desc,
    this.phone = "", this.www = "", this.instagram = "", this.telegram = "", required  this.descTitle,
    this.imageUpperServerPath = "", this.imageUpperLocalFile = "",
    this.logoServerPath = "", this.logoLocalFile = "", required this.gallery, required this.workTime,
    required this.category,
    this.login = "", required this.route, this.available = true,
    this.assetUpperImage = "", this.assetsLogo = "", required this.assetsGallery, required this.assetsCategory,
    required this.addon, this.tax = 10, required this.articles,
    //
    this.useMaximumServices = false, this.maxServices = 0, this.useMaxAddonInOneService = false,
    this.maxAddonInOneService = 0, this.useMaximumProducts = false, this.maxProducts = 0,
    this.useMinPurchaseAmount = false, this.minPurchaseAmount = 0, this.useMaxPurchaseAmount = false,
    this.maxPurchaseAmount = 0, this.acceptPaymentInCash = true,
  });

  factory ProviderData.createEmpty(){
    return ProviderData(descTitle: [StringData(code: "en", text: "Description",)], route: [], addon: [],
        gallery: [], workTime: [], desc: [], category: [], name: [], assetsGallery: [], assetsCategory: [],
        articles: []);
  }

  Map<String, dynamic> toJson() => {
    'name': name.map((i) => i.toJson()).toList(),
    'phone': phone,
    'www': www,
    'instagram': instagram,
    'telegram': telegram,
    'desc': desc.map((i) => i.toJson()).toList(),
    'descTitle': descTitle.map((i) => i.toJson()).toList(),
    'address': address,
    'visible': visible,
    'imageUpperServerPath': imageUpperServerPath,
    'imageUpperLocalFile': imageUpperLocalFile,
    'logoServerPath': logoServerPath,
    'logoLocalFile': logoLocalFile,
    'gallery': gallery.map((i) => i.toJson()).toList(),
    'workTime': workTime.map((i) => i.toJson()).toList(),
    'category': category,
    "login" : login,
    "tax" : tax,
    "route" : route.map((i){
      return {'lat': i.latitude, 'lng': i.longitude};
    }).toList(),
    'available': available,
    'addon': addon.map((i) => i.toJson()).toList(),
    'articles': articles,
    //
    'useMaximumServices': useMaximumServices,
    'maxServices': maxServices,
    'useMaxAddonInOneService': useMaxAddonInOneService,
    'maxAddonInOneService': maxAddonInOneService,
    'useMaximumProducts': useMaximumProducts,
    'maxProducts': maxProducts,
    'useMinPurchaseAmount': useMinPurchaseAmount,
    'minPurchaseAmount': minPurchaseAmount,
    'useMaxPurchaseAmount': useMaxPurchaseAmount,
    'maxPurchaseAmount': maxPurchaseAmount,
    'acceptPaymentInCash': acceptPaymentInCash,
  };

  factory ProviderData.fromJson(String id, Map<String, dynamic> data){
    List<String> _category = [];
    if (data['category'] != null)
      for (var element in List.from(data['category'])) {
        _category.add(element);
      }
    List<ImageData> _gallery = [];
    if (data['gallery'] != null)
      for (var element in List.from(data['gallery'])) {
        _gallery.add(ImageData(serverPath: element["serverPath"], localFile: element["localFile"]));
      }
    //
    List<WorkTimeData> _workTime = [];
    if (data['workTime'] != null)
      for (var element in List.from(data['workTime'])) {
        _workTime.add(WorkTimeData.fromJson(element));
      }
    //
    List<StringData> _name = [];
    if (data['name'] != null)
      for (var element in List.from(data['name'])) {
        _name.add(StringData.fromJson(element));
      }
    List<StringData> _desc = [];
    if (data['desc'] != null)
      for (var element in List.from(data['desc'])) {
        _desc.add(StringData.fromJson(element));
      }
    List<StringData> _descTitle = [];
    if (data['descTitle'] != null)
      for (var element in List.from(data['descTitle'])) {
        _descTitle.add(StringData.fromJson(element));
      }
    List<LatLng> _route = [];
    if (data["route"] != null)
      for (var element in List.from(data['route'])) {
        if (element['lat'] != null && element['lng'] != null)
          _route.add(LatLng(
              element['lat'], element['lng']
          ));
      }
    List<AddonData> _addon = [];
    if (data['addon'] != null)
      for (var element in List.from(data['addon'])) {
        _addon.add(AddonData.fromJson(element));
      }
    List<String> _articles = [];
    if (data['articles'] != null)
      for (var element in List.from(data['articles'])) {
        _articles.add(element);
      }
    return ProviderData(
      id: id,
      name: _name,
      phone: (data["phone"] != null) ? data["phone"] : "",
      www: (data["www"] != null) ? data["www"] : "",
      instagram: (data["instagram"] != null) ? data["instagram"] : "",
      telegram: (data["telegram"] != null) ? data["telegram"] : "",
      desc: _desc,
      descTitle: _descTitle,
      address: (data["address"] != null) ? data["address"] : "",
      visible: (data["visible"] != null) ? data["visible"] : true,
      imageUpperServerPath: (data["imageUpperServerPath"] != null) ? data["imageUpperServerPath"] : "",
      imageUpperLocalFile: (data["imageUpperLocalFile"] != null) ? data["imageUpperLocalFile"] : "",
      logoServerPath: (data["logoServerPath"] != null) ? data["logoServerPath"] : "",
      logoLocalFile: (data["logoLocalFile"] != null) ? data["logoLocalFile"] : "",
      gallery: _gallery,
      workTime: _workTime,
      category: _category,
      //avatar: (data["avatar"] != null) ? data["avatar"] : "",
      login: (data["login"] != null) ? data["login"] : "",
      route : _route,
      available: (data["available"] != null) ? data["available"] : true,
      addon: _addon,
      tax: (data["tax"] != null) ? toDouble(data["tax"].toString()) : 10.0,
      assetsCategory: [],
      assetsGallery: [],
      articles: _articles,
      //
      useMaximumServices: (data["useMaximumServices"] != null) ? data["useMaximumServices"] : false,
      maxServices: (data["maxServices"] != null) ? toInt(data["maxServices"].toString()) : 0,
      useMaxAddonInOneService: (data["useMaxAddonInOneService"] != null) ? data["useMaxAddonInOneService"] : false,
      maxAddonInOneService: (data["maxAddonInOneService"] != null) ? toInt(data["maxAddonInOneService"].toString()) : 0,
      useMaximumProducts: (data["useMaximumProducts"] != null) ? data["useMaximumProducts"] : false,
      maxProducts: (data["maxProducts"] != null) ? toInt(data["maxProducts"].toString()) : 0,
      useMinPurchaseAmount: (data["useMinPurchaseAmount"] != null) ? data["useMinPurchaseAmount"] : false,
      minPurchaseAmount: (data["minPurchaseAmount"] != null) ? toDouble(data["minPurchaseAmount"].toString()) : 0.0,
      useMaxPurchaseAmount: (data["useMaxPurchaseAmount"] != null) ? data["useMaxPurchaseAmount"] : false,
      maxPurchaseAmount: (data["maxPurchaseAmount"] != null) ? toDouble(data["maxPurchaseAmount"].toString()) : 0.0,
      acceptPaymentInCash: (data["acceptPaymentInCash"] != null) ? data["acceptPaymentInCash"] : true,
    );
  }

  compareToAvailable(ProviderData b){
    if (available && !b.available)
      return 1;
    if (!available && b.available)
      return -1;
    return 0;
  }
}

class WorkTimeData{
  int id = 0;
  bool weekend = false;
  String openTime = "";
  String closeTime = "";

  WorkTimeData({this.id = 0, this.openTime = "09:00", this.closeTime = "16:00", this.weekend = false});

  Map<String, dynamic> toJson() => {
    'index': id,
    'openTime': openTime,
    'closeTime': closeTime,
    'weekend': weekend,
  };

  factory WorkTimeData.fromJson(Map<String, dynamic> data){
    return WorkTimeData(
      id: (data["index"] != null) ? data["index"] : 0,
      openTime: (data["openTime"] != null) ? data["openTime"] : "9:00",
      closeTime: (data["closeTime"] != null) ? data["closeTime"] : "16:00",
      weekend: (data["weekend"] != null) ? data["weekend"] : false,
    );
  }

  factory WorkTimeData.createEmpty(){
    return WorkTimeData();
  }
}
